import json
from dataclasses import dataclass
from pathlib import Path

import nbformat

from ssb_pubmd.models import Content

ID_KEY = "_id"


class LocalStorageError(Exception): ...


@dataclass
class LocalStorageAdapter:
    metadata_file_path: Path

    def get_file_id(self, file_path: Path) -> str:
        """
        Returns the content id of a given file path.
        If no id is registered for the given file path,
        it returns an empty string.
        """
        with open(self.metadata_file_path) as metadata_file:
            metadata = json.load(metadata_file)
            return str(metadata.get(str(file_path.absolute()), {}).get(ID_KEY, ""))

    def set_file_id(self, file_path: Path, content_id: str) -> None:
        """Stores a given file's content id in the metadata file"""
        with open(self.metadata_file_path) as metadata_file:
            metadata = json.load(metadata_file)
        metadata[str(file_path.absolute())] = {ID_KEY: content_id}
        with open(self.metadata_file_path, "w") as metadata_file:
            json.dump(metadata, metadata_file)

    def get_content(self, file_path: Path) -> Content:
        """
        Returns the markdown content of a given file.
        If the file is neither a .md or a .ipynb file,
        this function will throw a `LocalStorageError`
        """
        return Content(
            content_id=self.get_file_id(file_path),
            file_path=file_path,
            markdown=self._get_content(file_path),
        )

    def _get_content_from_notebook_file(self, file_path: Path) -> str:
        """Extracts all markdown cells from the notebook and returns it as a string."""
        notebook = nbformat.read(file_path, as_version=nbformat.NO_CONVERT)  # type: ignore
        markdown_cells = []
        for cell in notebook.cells:
            if cell.cell_type == "markdown":
                markdown_cells.append(cell.source)
        sep = "\n\n"
        return sep.join(markdown_cells)

    def _get_content_from_markdown_file(self, file_path: Path) -> str:
        """Returns the content of a markdown file as a string."""
        with open(file_path) as file:
            return file.read()

    def _get_content(self, file_path: Path) -> str:
        file_type = file_path.suffix
        match file_type:
            case ".md":
                return self._get_content_from_markdown_file(file_path)
            case ".ipynb":
                return self._get_content_from_notebook_file(file_path)
            case _:
                raise LocalStorageError(f"Unsupported file type: {file_type}")



"""This module handles HTTP requests and responses to and from the CMS."""

import json
from dataclasses import dataclass
from datetime import datetime

import jwt
from google.cloud import secretmanager


@dataclass
class Secret:
    private_key: str
    kid: str
    principal_key: str


class SecretManagerError(Exception): ...


class GoogleSecretManagerClient:
    TYPE = "JWT"
    ALGORITHM = "RS256"
    _gc_secret_resource_name: str

    def __init__(self, gc_secret_resource_name: str) -> None:
        self._gc_secret_resource_name = gc_secret_resource_name

    def _get_secret(self) -> Secret:
        """Fetches the private key and related data from Google Cloud Secret Manager."""
        client = secretmanager.SecretManagerServiceClient()
        response = client.access_secret_version(name=self._gc_secret_resource_name)
        raw_data = response.payload.data.decode("UTF-8")
        data = json.loads(raw_data)
        try:
            return Secret(
                private_key=data["privateKey"],
                kid=data["kid"],
                principal_key=data["principalKey"],
            )
        except KeyError as e:
            raise SecretManagerError(
                "The secret must be a JSON object with keys 'privateKey', 'kid' and 'principalKey'."
            ) from e

    def generate_token(self) -> str:
        secret = self._get_secret()

        header = {
            "kid": secret.kid,
            "typ": self.TYPE,
            "alg": self.ALGORITHM,
        }

        iat = int(datetime.now().timestamp())
        exp = iat + 30
        payload = {
            "sub": secret.principal_key,
            "iat": iat,
            "exp": exp,
        }

        token = jwt.encode(
            payload, secret.private_key, algorithm=self.ALGORITHM, headers=header
        )
        return token



import sys

from ssb_pubmd.adapters.cli import CliAdapter
from ssb_pubmd.adapters.cms_client import MimirCmsClient
from ssb_pubmd.adapters.local_storage import LocalStorageAdapter
from ssb_pubmd.adapters.secret_manager_client import GoogleSecretManagerClient
from ssb_pubmd.config import get_config
from ssb_pubmd.enonic_cms_manager import EnonicCmsManager


def main() -> None:
    config = get_config()
    cms_manager = EnonicCmsManager(
        config=config,
        cms_client=MimirCmsClient(config.cms_base_url),
        secret_manager_client=GoogleSecretManagerClient(config.gc_secret_resource_name),
        content_file_handler=LocalStorageAdapter(config.metadata_file_path),
    )
    cli_adapter = CliAdapter(cms_manager=cms_manager)
    cli_adapter.run(sys.argv)


if __name__ == "__main__":
    main()



from urllib.parse import urlparse

import requests

from ssb_pubmd.models import CmsResponse
from ssb_pubmd.models import Content


class CmsClientError(Exception): ...


class MimirCmsClient:
    base_url: str

    def __init__(self, base_url: str) -> None:
        self.base_url = base_url

    def _convert_preview_url(self, url_from_response: str) -> str:
        """Convert the preview URL to a full URL if it's relative."""
        url = urlparse(url_from_response)
        if url.scheme and url.netloc:
            return url.geturl()
        else:
            return urlparse(self.base_url)._replace(path=url.path).geturl()

    def send(self, token: str, content: Content) -> CmsResponse:
        """Sends a request to the Enonic CMS, assumed to have the mimir application installed (currently this only works with the feature branch https://github.com/statisticsnorway/mimir/pull/3192)."""
        try:
            response = requests.post(
                f"{self.base_url}/_/service/mimir/postMarkdown",
                headers={
                    "Authorization": f"Bearer {token}",
                    "Content-Type": "application/json",
                },
                json=content.to_json(),
            )
            if response.status_code != 200:
                raise CmsClientError(
                    f"Request to CMS failed with status code {response.status_code}."
                )
            body = response.json()
            return CmsResponse(
                id=body["_id"],
                preview_url=self._convert_preview_url(body["previewPath"]),
            )
        except Exception as e:
            raise CmsClientError("Request to CMS failed.") from e



from pathlib import Path
from typing import Protocol

from ssb_pubmd.models import CmsResponse
from ssb_pubmd.models import Content


class CmsManager(Protocol):
    """An interface (primary port) for managing a CMS.

    An implementing class is tailored to a specific CMS application.
    """

    def sync(self, content_file_path: str) -> str:
        """Requests that the CMS stores and renders the given content file.

        :param content_file_path: The path to the content file to be rendered.
        :return: A URL to the rendered preview.
        """
        ...


class CmsClient(Protocol):
    """An interface (secondary port) for communicating with a CMS.

    An implementing class is tailored to a specific CMS, with its specific exposed services and token authentication flow.
    """

    def send(self, token: str, content: Content) -> CmsResponse: ...


class ContentFileHandler(Protocol):
    """An interface (secondary port) for handling content files, including extraction and handling of metadata."""

    def get_file_id(self, file_path: Path) -> str:
        """
        Returns the id of a given file path.
        If no id is registered for the given file path,
        it returns an empty string.
        """
        ...

    def set_file_id(self, file_path: Path, content_id: str) -> None:
        """Sets the id of a given file path."""
        ...

    def get_content(self, file_path: Path) -> Content:
        """
        Extracts the content of a given file.
        """
        ...


class SecretManagerClient(Protocol):
    """An interface (secondary port) for communicating with a secret manager and generating tokens."""

    def generate_token(self) -> str: ...



from dataclasses import dataclass
from pathlib import Path


@dataclass
class CmsResponse:
    id: str
    preview_url: str


@dataclass
class Content:
    content_id: str
    file_path: Path
    markdown: str

    @property
    def display_name(self) -> str:
        """Generate a display name for the content."""
        return self.file_path.stem.replace("_", " ").title()

    def to_json(self) -> dict[str, str]:
        """Returns a json representation of the content."""
        return {
            "_id": self.content_id,
            "displayName": self.display_name,
            "markdown": self.markdown,
        }



import sys
from dataclasses import dataclass

from ssb_pubmd.ports import CmsManager


@dataclass
class CliAdapter:
    cms_manager: CmsManager

    def run(self, system_arguments: list[str]) -> None:
        match system_arguments:
            case [_, "sync", file_path]:
                preview = self.cms_manager.sync(file_path)
                print(f"Preview URL: {preview}")
                # except Exception as e:
                #     print(f"Error during sync: {e}")
                #     sys.exit(1)
            case _:
                print("Usage: ssb-pubmd sync <content_file_path>")
                sys.exit(1)



import os
from dataclasses import dataclass
from pathlib import Path

APP_NAME = "SSB_PUBMD"


@dataclass
class Config:
    metadata_file_path: Path
    cms_base_url: str
    gc_secret_resource_name: str


def get_config(metadata_file_path: Path | None = None) -> Config:
    user_data_dir = Path.home() / ".local" / "share" / APP_NAME
    if not metadata_file_path:
        user_data_dir.mkdir(parents=True, exist_ok=True)
        metadata_file_path = user_data_dir / "metadata.json"
    if not metadata_file_path.exists():
        with open(metadata_file_path, "x") as metadata_file:
            metadata_file.write("{}\n")
    return Config(
        metadata_file_path=metadata_file_path,
        cms_base_url=os.environ[f"{APP_NAME}_BASE_URL"],
        gc_secret_resource_name=os.environ[f"{APP_NAME}_GC_SECRET_RESOURCE_NAME"],
    )



from dataclasses import dataclass
from pathlib import Path

from ssb_pubmd.config import Config
from ssb_pubmd.ports import CmsClient
from ssb_pubmd.ports import ContentFileHandler
from ssb_pubmd.ports import SecretManagerClient


@dataclass
class EnonicCmsManager:
    """A CMS Mananager tailored to the Enonic CMS."""

    config: Config
    cms_client: CmsClient
    secret_manager_client: SecretManagerClient
    content_file_handler: ContentFileHandler

    def sync(self, content_file_path: str) -> str:
        """Requests that Enonic stores/updates the given contant file and gives back a rendered preview.

        The details of the communication are handled by the CmsClient implementation, which in turn depends on the services that are exposed by the Enonic XP application. The only thing this class cares is that it receives a CmsResponse object, which contains an id and preview url of the content.
        """
        content = self.content_file_handler.get_content(Path(content_file_path))
        response = self.cms_client.send(
            token=self.secret_manager_client.generate_token(),
            content=content,
        )
        self.content_file_handler.set_file_id(content.file_path, response.id)
        return response.preview_url
